-----------------------------------------------------------------
-- Fichier contenant les specification VHDL de synthese
-- des operateurs de base :
--
-- 	reg : registre 16 bits (avec load synchrone et reset asynchrone) 
-- 	mult : multiplieur 8x8 -> 16
-- 	addsous : additionneur / soustracteur (sans gestion du debordement)
-- 	tri : tristate 16 bits a commande /OE
-- 	mux : multiplexeur 16 bits 
--	racine : calcul de la racine 16b -> 8b
-----------------------------------------------------------------

-------------------------------------------------------------------------
--  REGISTRE 16 bits  *CLOCK ACTIVE SUR FRONT MONTANT
--                    *CLEAR ASYNCHRONE ACTIF A '0'
--		      *LOAD SYNCHRONE ACTIF A '1'
-------------------------------------------------------------------------    
library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity reg is
    port ( D : in Std_Logic_Vector(15 downto 0);
           LOAD : in Std_Logic;
           CLRB : in Std_Logic;
           CLK : in Std_Logic;
           Q : out Std_Logic_Vector(15 downto 0) );
end reg;

architecture reg OF reg is begin
    process(CLRB,CLK)
        begin
        if CLRB='0' then 
		Q <="0000000000000000";   -- si clear=0 alors Q=0
         elsif CLK'event and CLK='1' then -- si front montant sur clk
                if LOAD='1' then Q <= D;  -- et load=1 alors chargement
                end if;
        end if;
   end process;
end reg;

-------------------------------------------------------------------------
--                    MULTIPLIEUR 8 bits                                -
--      Entrees : A entree sur 8 b                                      -
--                B entree sur 8 b                                      -
--      Sorties : S sortie sur 16 b                                     -
-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_arith.all;

entity mult is

    port ( A : in Std_Logic_Vector(7 downto 0);
           B : in Std_Logic_Vector(7 downto 0);
           S : out Std_Logic_Vector(15 downto 0) );
end mult;

architecture mult OF mult is

begin
  process(A,B)
  variable va_A_ext : signed(15 downto 0);
  variable va_B_ext : signed(15 downto 0);
  variable tmp      : signed(31 downto 0);
  
  begin
  	va_A_ext := conv_signed(signed(A),16);
  	va_B_ext := conv_signed(signed(B),16);
        tmp := va_A_ext * va_B_ext;
	S <= std_logic_vector(tmp(15 downto 0));
  end process;
end mult;


-------------------------------------------------------------------------
--		ADDITIONNEUR SOUSTRACTEUR 			        -
-- 									-
-- programme VHDL permettant l'addition/soustraction de A et B		-
--    Si commande = 0 alors S=A+B					-
--    Si commande = 1 alors S=A-B					-
-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_arith.all;


entity addsous is
    port ( A : in Std_Logic_Vector(15 downto 0) ;
           B : in Std_Logic_Vector(15 downto 0) ;
           COMMANDE : in BIT ;
           S : out Std_Logic_Vector(15 downto 0) ) ;
end addsous;

architecture addsous OF addsous is

begin
    process(A,B,COMMANDE)
        begin
        if COMMANDE = '0' then
                S <= signed(A) + signed(B);
        else
                S <= signed(A) - signed(B);
        end if;
    end process;
end addsous;


---------------------------------------------------------------------
-- 	 	TRI-STATE				
--              OE  D      Q
--              0   X  ->  Z
--              1   0  ->  0
--              1   1  ->  1
----------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity tri is
    port ( D : in Std_Logic_Vector(15 downto 0) ;
           OE : in Std_Logic ;
           Q : out Std_Logic_Vector(15 downto 0) );
end tri;

architecture tri OF tri is begin
    process(D,OE)
    begin
        if OE = '0' then
                Q <= "ZZZZZZZZZZZZZZZZ";
        else
                Q <= D;
        end if;
    end process;
end tri;

-------------------------------------------------------------------------
--                       MULTIPLEXEUR 16 b                              -
--      Entrees: E0,E1 sur 16 b                                         -
--      Sortie: S du mux sur 16 b                                       -
--                                                                      -
--      Table de verite:                                                -
--                      COMMANDE | S                                    -
--                          0    | E0                                   -
--                          1    | E1                                   -
-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity mux is
    port ( E0 : in Std_Logic_Vector(15 downto 0);
           E1 : in Std_Logic_Vector(15 downto 0);
           COMMANDE : in Std_Logic;
           S : out Std_Logic_Vector(15 downto 0) );
end mux;

architecture mux OF mux is begin
    process(E0,E1,COMMANDE)
        begin
        if COMMANDE='0' then 
	    S <=E0;
        else
	    S <=E1;
        end if;
    end process;
end mux;

-------------------------------------------------------------------------
--                       RACINE 16 b                              
--      Entrees: Valeur sur 16 b                                         
--      Sortie: S sur 8 b                                       
--                                                                      
-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;


ENTITY racine IS
 	PORT (Valeur : IN INTEGER RANGE 0 TO 16383;
	      Racine : OUT INTEGER RANGE 0 TO 127 );
END racine;

ARCHITECTURE racine of racine IS
	BEGIN

	PROCESS (Valeur) BEGIN
	case Valeur is
         	When  0 => Racine <= 0 ;
                When 1 To 2 => Racine <= 1 ;
                When 3 To 6 => Racine <= 2 ;
                When 7 To 12 => Racine <= 3 ;
                When 13 To 20 => Racine <= 4 ;
                When 21 To 30 => Racine <= 5 ;
                When 31 To 42 => Racine <= 6 ;
                When 43 To 56 => Racine <= 7 ;
                When 57 To 72 => Racine <= 8 ;
                When 73 To 90 => Racine <= 9 ;
                When 91 To 110 => Racine <= 10 ;
                When 111 To 132 => Racine <= 11 ;
                When 133 To 156 => Racine <= 12 ;
                When 157 To 182 => Racine <= 13 ;
                When 183 To 210 => Racine <= 14 ;
                When 211 To 240 => Racine <= 15 ;
                When 241 To 272 => Racine <= 16 ;
                When 273 To 306 => Racine <= 17 ;
                When 307 To 342 => Racine <= 18 ;
                When 343 To 380 => Racine <= 19 ;
                When 381 To 420 => Racine <= 20 ;
                When 421 To 462 => Racine <= 21 ;
                When 463 To 506 => Racine <= 22 ;
                When 507 To 552 => Racine <= 23 ;
                When 553 To 600 => Racine <= 24 ;
                When 601 To 650 => Racine <= 25 ;
                When 651 To 702 => Racine <= 26 ;
                When 703 To 756 => Racine <= 27 ;
                When 757 To 812 => Racine <= 28 ;
                When 813 To 870 => Racine <= 29 ;
                When 871 To 930 => Racine <= 30 ;
                When 931 To 992 => Racine <= 31 ;
                When 993 To 1056 => Racine <= 32 ;
                When 1057 To 1122 => Racine <= 33 ;
                When 1123 To 1190 => Racine <= 34 ;
                When 1191 To 1260 => Racine <= 35 ;
                When 1261 To 1332 => Racine <= 36 ;
                When 1333 To 1406 => Racine <= 37 ;
                When 1407 To 1482 => Racine <= 38 ;
                When 1483 To 1560 => Racine <= 39 ;
                When 1561 To 1640 => Racine <= 40 ;
                When 1641 To 1722 => Racine <= 41 ;
                When 1723 To 1806 => Racine <= 42 ;
                When 1807 To 1892 => Racine <= 43 ;
                When 1893 To 1980 => Racine <= 44 ;
                When 1981 To 2070 => Racine <= 45 ;
                When 2071 To 2162 => Racine <= 46 ;
                When 2163 To 2256 => Racine <= 47 ;
                When 2257 To 2352 => Racine <= 48 ;
                When 2353 To 2450 => Racine <= 49 ;
                When 2451 To 2550 => Racine <= 50 ;
                When 2551 To 2652 => Racine <= 51 ;
                When 2653 To 2756 => Racine <= 52 ;
                When 2757 To 2862 => Racine <= 53 ;
                When 2863 To 2970 => Racine <= 54 ;
                When 2971 To 3080 => Racine <= 55 ;
                When 3081 To 3192 => Racine <= 56 ;
                When 3193 To 3306 => Racine <= 57 ;
                When 3307 To 3422 => Racine <= 58 ;
                When 3423 To 3540 => Racine <= 59 ;
                When 3541 To 3660 => Racine <= 60 ;
                When 3661 To 3782 => Racine <= 61 ;
                When 3783 To 3906 => Racine <= 62 ;
                When 3907 To 4032 => Racine <= 63 ;
                When 4033 To 4160 => Racine <= 64 ;
                When 4161 To 4290 => Racine <= 65 ;
                When 4291 To 4422 => Racine <= 66 ;
                When 4423 To 4556 => Racine <= 67 ;
                When 4557 To 4692 => Racine <= 68 ;
                When 4693 To 4830 => Racine <= 69 ;
                When 4831 To 4970 => Racine <= 70 ;
                When 4971 To 5112 => Racine <= 71 ;
                When 5113 To 5256 => Racine <= 72 ;
                When 5257 To 5402 => Racine <= 73 ;
                When 5403 To 5550 => Racine <= 74 ;
                When 5551 To 5700 => Racine <= 75 ;
                When 5701 To 5852 => Racine <= 76 ;
                When 5853 To 6006 => Racine <= 77 ;
                When 6007 To 6162 => Racine <= 78 ;
                When 6163 To 6320 => Racine <= 79 ;
                When 6321 To 6480 => Racine <= 80 ;
                When 6481 To 6642 => Racine <= 81 ;
                When 6643 To 6806 => Racine <= 82 ;
                When 6807 To 6972 => Racine <= 83 ;
                When 6973 To 7140 => Racine <= 84 ;
                When 7141 To 7310 => Racine <= 85 ;
                When 7311 To 7482 => Racine <= 86 ;
                When 7483 To 7656 => Racine <= 87 ;
                When 7657 To 7832 => Racine <= 88 ;
                When 7833 To 8010 => Racine <= 89 ;
                When 8011 To 8190 => Racine <= 90 ;
                When 8191 To 8372 => Racine <= 91 ;
                When 8373 To 8556 => Racine <= 92 ;
                When 8557 To 8742 => Racine <= 93 ;
                When 8743 To 8930 => Racine <= 94 ;
                When 8931 To 9120 => Racine <= 95 ;
                When 9121 To 9312 => Racine <= 96 ;
                When 9313 To 9506 => Racine <= 97 ;
                When 9507 To 9702 => Racine <= 98 ;
                When 9703 To 9900 => Racine <= 99 ;
                When 9901 To 10100 => Racine <= 100 ;
                When 10101 To 10302 => Racine <= 101 ;
                When 10303 To 10506 => Racine <= 102 ;
                When 10507 To 10712 => Racine <= 103 ;
                When 10713 To 10920 => Racine <= 104 ;
                When 10921 To 11130 => Racine <= 105 ;
                When 11131 To 11342 => Racine <= 106 ;
                When 11343 To 11556 => Racine <= 107 ;
                When 11557 To 11772 => Racine <= 108 ;
                When 11773 To 11990 => Racine <= 109 ;
                When 11991 To 12210 => Racine <= 110 ;
                When 12211 To 12432 => Racine <= 111 ;
                When 12433 To 12656 => Racine <= 112 ;
                When 12657 To 12882 => Racine <= 113 ;
                When 12883 To 13110 => Racine <= 114 ;
                When 13111 To 13340 => Racine <= 115 ;
                When 13341 To 13572 => Racine <= 116 ;
                When 13573 To 13806 => Racine <= 117 ;
                When 13807 To 14042 => Racine <= 118 ;
                When 14043 To 14280 => Racine <= 119 ;
                When 14281 To 14520 => Racine <= 120 ;
                When 14521 To 14762 => Racine <= 121 ;
                When 14763 To 15006 => Racine <= 122 ;
                When 15007 To 15252 => Racine <= 123 ;
                When 15253 To 15500 => Racine <= 124 ;
                When 15501 To 15750 => Racine <= 125 ;
                When 15751 To 16002 => Racine <= 126 ;
                When 16003 To 16383 => Racine <= 127 ;
		when others => Racine <= 0 ;
		end case; 
	END PROCESS;
END racine ;
